@@ -1,7 +1,7 @@
 /* kplex: An anything to anything boat data multiplexer for Linux
  * Currently this program only supports nmea-0183 data.
  * For currently supported interfaces see kplex_mods.h
- * Copyright Keith Young 2012-2016
+ * Copyright Keith Young 2012-2017
  * For copying information, see the file COPYING distributed with this file
  */
 
@@ -20,10 +20,12 @@
 #include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/time.h>
+#include <sys/wait.h>
 #include <inttypes.h>
+#include <fcntl.h>
 
 /* Macro to identify kplex Proprietary sentences */
-#define isprop(sptr) (sptr->data[1] == 'P' && sptr->data[2] == 'K' && sptr->data[3] == 'P' && sptr->data[4] == 'X')
+#define isprop(sptr) (sptr->len >= 7 && sptr->data[1] == 'P' && sptr->data[2] == 'K' && sptr->data[3] == 'P' && sptr->data[4] == 'X')
 
 /* Globals. Sadly. Used in signal handlers so few other simple options */
 pthread_key_t ifkey;    /* Key for Thread local pointer to interface struct */
@@ -70,12 +72,12 @@ int checkcksum (senblk_t *sptr)
     int rcvdcksum=0,i,end;
     char *ptr;
 
-    for(i=0,end=sptr->len-5,ptr=sptr->data+1;*ptr != '*'; ptr++,i++)
-        if (i == end)
-            return(-1);
-        else
+    for(i=0,end=sptr->len-6,ptr=sptr->data+1; i < end; ptr++,i++)
             cksm ^= *ptr;
 
+    if (*ptr != '*')
+        return -1;
+
     for (i=0,++ptr;i<2;i++,ptr++) {
         if (*ptr>47 && *ptr<58)
             rcvdcksum+=*ptr-48;
@@ -90,7 +92,7 @@ int checkcksum (senblk_t *sptr)
     if (cksm == rcvdcksum)
         return (0);
     else
-        return(1);
+        return(-1);
 }
 
 /*
@@ -311,7 +313,7 @@ int addfailover(sfilter_t **head,char *spec)
         src->lasttime=now;
         link_src_to_rule(&newrule->info.source,src);
     }
-    if (done)
+    if (done) {
         if (!*head) {
             if (((*head)=(sfilter_t *)malloc(sizeof(sfilter_t)))) {
                 (*head)->type=FAILOVER;
@@ -324,6 +326,7 @@ int addfailover(sfilter_t **head,char *spec)
             newrule->next=(*head)->rules;
             (*head)->rules=newrule;
             return(0);
+        }
     }
     if (src)
         free(src);
@@ -364,7 +367,9 @@ int init_q(iface_t *ifa, size_t size)
     if ((newq=(ioqueue_t *)malloc(sizeof(ioqueue_t))) == NULL)
         return(-1);
     if ((newq->base=(senblk_t *)calloc(size,sizeof(senblk_t))) ==NULL) {
+        i=errno;
         free(newq);
+        errno=i;
         return(-1);
     }
 
@@ -430,7 +435,7 @@ void push_senblk(senblk_t *sptr, ioqueue_t *q)
             q->qhead=q->qhead->next;
             if (q->drops < 0)
                 q->drops++;
-            DEBUG(4,"Dropped senblk q=0x%x",q);
+            DEBUG(4,"Dropped senblk q=%s",(q->owner->name)?q->owner->name:"(unknown)");
         }
     
         (void) senblk_copy(tptr,sptr);
@@ -574,7 +579,8 @@ iface_t *get_default_global()
     }
     ifg->flags=0;
     ifg->logto=LOG_DAEMON;
-    ifp->strict=1;
+    ifp->strict=-1;
+    ifp->checksum=0;
     ifp->info = (void *)ifg;
 
     return(ifp);
@@ -950,6 +956,41 @@ char *get_def_config()
         }
         strcpy(buf,confptr);
         strcat(buf,"/");
+
+#ifdef KPLEXHOMECONFOSX
+/*
+ * Deprecate OSX specific config file.  This seemed like a good idea at the time
+ * but has no apparent advantage  over ~/.kplex.conf
+ */
+        int doosxconf=1;
+        size_t osxlen,baselen;
+
+        if ((osxlen=strlen(KPLEXHOMECONFOSX)) <
+                (baselen=strlen(KPLEXOLDHOMECONFOSX))) {
+            osxlen=baselen;
+        }
+
+        if ((strlen(KPLEXHOMECONF)) < osxlen) {
+            if (realloc(buf,(baselen=strlen(buf))+osxlen+1) == NULL) {
+                perror("Can't query OSX config file");
+                doosxconf=0;
+            }
+        }
+
+        if (doosxconf) {
+            strcat(buf,KPLEXOLDHOMECONFOSX);
+            if (!access(buf,F_OK)) {
+                logwarn("Use of %s is deprecated for kplex config.\nPlease move this file to ~/%s to suppress this warning",buf,KPLEXHOMECONF);
+                return(buf);
+            }
+            strcpy(buf+baselen,KPLEXHOMECONFOSX);
+            if (!access(buf,F_OK)) {
+                return(buf);
+            }
+            buf[baselen]='\0';
+        }
+#endif
+
         strcat(buf,KPLEXHOMECONF);
         if (!access(buf,F_OK))
             return(buf);
@@ -1023,6 +1064,7 @@ int name2id(sfilter_t *filter)
                 return(-1);
             }
             free(rptr->src.name);
+            rptr->src.name=NULL;
             rptr->src.id=id;
         }
         return(0);
@@ -1035,6 +1077,7 @@ int name2id(sfilter_t *filter)
                 return(-1);
             }
             free(sptr->src.name);
+            sptr->src.name=NULL;
             sptr->src.id=id;
         }
     return(0);
@@ -1043,7 +1086,7 @@ int name2id(sfilter_t *filter)
 int proc_engine_options(iface_t *e_info,struct kopts *options)
 {
     struct kopts *optr;
-    size_t qsize=DEFQUEUESZ;
+    size_t qsize=DEFQSIZE;
     struct if_engine *ifg = (struct if_engine *) e_info->info;
 
     if (e_info->options) {
@@ -1071,6 +1114,13 @@ int proc_engine_options(iface_t *e_info,struct kopts *options)
                 fprintf(stderr,"Unknown log facility \'%s\' specified\n",optr->val);
                 exit(1);
             }
+        } else if (!strcasecmp(optr->var,"debuglevel")) {
+            if ((*optr->val < '0') || (*optr->val > '9') || *(optr->val+1)) {
+                fprintf(stderr,"Bad debug level \"%s\": Must be 0-9\n",optr->val);
+                exit(0);
+            } else {
+                debuglevel = (int) (*optr->val - '0');
+            }
         } else if (!strcasecmp(optr->var,"graceperiod")) {
             if (((graceperiod=(time_t) strtoumax(optr->val,NULL,0)) == 0) &&
                     (errno)) {
@@ -1159,7 +1209,7 @@ size_t gettag(iface_t *ifa, char *buf, senblk_t *sptr)
         (void) gettimeofday(&tv,NULL);
         ptr+=sprintf(ptr,"%010u",(unsigned) tv.tv_sec);
         if (ifa->tagflags & TAG_MS)
-            ptr += sprintf(ptr,"%03u",((unsigned) tv.tv_usec+500)/1000);
+            ptr += sprintf(ptr,"%03u",((unsigned) tv.tv_usec)/1000);
     }
     /* Don't include initial '/' */
     cksum=calcsum(buf+1,(len=ptr-buf)-1);
@@ -1236,6 +1286,9 @@ void do_read(iface_t *ifa)
                     senstate = SEN_NODATA;
                     continue;
                 }
+                /* If we're not checksumming OR the checksum is correct OR
+                 * it's a zero length packet, the first clause is false which
+                 * is true when negated...*/
                 if (!(ifa->checksum && checkcksum(&sblk) && (sblk.len > 0 )) &&
                         senfilter(&sblk,ifa->ifilter) == 0) {
                     push_senblk(&sblk,ifa->q);
@@ -1303,10 +1356,12 @@ char * mkname(iface_t *ifa, unsigned int i)
     
 int main(int argc, char ** argv)
 {
-    long templ;
+    char *tmpbuf;
     pthread_t tid;
     pid_t pid;
+    int pfd;
     char *config=NULL;
+    char *pidfile=NULL;
     iface_t  *engine;
     struct if_engine *ifg;
     iface_t *ifptr,*ifptr2,*rptr;
@@ -1315,7 +1370,7 @@ int main(int argc, char ** argv)
     int opt,err=0;
     void *ret;
     struct kopts *options=NULL;
-    sigset_t set;
+    sigset_t set,oset;
     struct iolists lists = {
         /* initialize io_mutex separately below */
         .init_mutex = PTHREAD_MUTEX_INITIALIZER,
@@ -1327,6 +1382,7 @@ int main(int argc, char ** argv)
     .dead = NULL
     };
     struct rlimit lim;
+    struct flock *fl;
     int gotinputs=0;
     int rcvdsig;
     struct sigaction sa;
@@ -1334,15 +1390,23 @@ int main(int argc, char ** argv)
     pthread_mutex_init(&lists.io_mutex,NULL);
 
     /* command line argument processing */
-    while ((opt=getopt(argc,argv,"d:f:o:V")) != -1) {
+    while ((opt=getopt(argc,argv,"d:f:o:p:V")) != -1) {
         switch (opt) {
             case 'd':
-                errno=0;
-                if (((templ=strtol(optarg,NULL,0)) <= 0) || templ > 9) {
-                    logerr(errno,"Bad debug level %s: Must be 1-9",optarg);
+                if (*(optarg+1)) {
+                    /* debuglevel is more than one character long */
+                    fprintf(stderr,"Bad debug level \"%s\": Must be 0-9\n",
+                            optarg);
                     err++;
-                } else
-                    debuglevel=templ;
+                } else if ((tmpbuf = (char *) malloc(13)) == NULL ) {
+                    logerr(errno,"failed to allocate memory");
+                    err++;
+                } else {
+                    sprintf(tmpbuf,"debuglevel=%c",*optarg);
+                    if (cmdlineopt(&options,tmpbuf) < 0)
+                        err++;
+                    free(tmpbuf);
+                }
                 break;
             case 'o':
                 if (cmdlineopt(&options,optarg) < 0)
@@ -1351,6 +1415,9 @@ int main(int argc, char ** argv)
             case 'f':
                 config=optarg;
                 break;
+            case 'p':
+                pidfile=optarg;
+                break;
             case 'V':
                 printf("%s\n",VERSION);
                 if (argc == 2)
@@ -1364,10 +1431,11 @@ int main(int argc, char ** argv)
     }
 
     if (err) {
-        fprintf(stderr, "Usage: %s [-V] | [ -f <config file>] [-o <option=value>]... [<interface specification> ...]\n",argv[0]);
+        fprintf(stderr, "Usage: %s [-V] | [ -d <level> ] [ -p <pid file> ] [ -f <config file>] [-o <option=value>]... [<interface specification> ...]\n",argv[0]);
         exit(1);
     }
 
+
     /* If a config file is specified by a commad line argument, read it.  If
      * not, look for a default config file unless told not to using "-f-" on the
      * command line
@@ -1408,16 +1476,72 @@ int main(int argc, char ** argv)
      * Advantage: We can close all the file descriptors now rather than pulling
      * then from under erroneously specified stdin/stdout etc.
      */
-
+    sigemptyset(&set);
+    sigaddset(&set, SIGCHLD);
+    sigaddset(&set, SIGUSR1);
+    sigprocmask(SIG_BLOCK,&set,&oset);
     ifg=(struct if_engine *)engine->info;
     if (ifg->flags & K_BACKGROUND) {
          if ((pid = fork()) < 0) {
             perror("fork failed");
             exit(1);
-        } else if (pid)
-            exit(0);
+        } else if (pid) {
+            sigwait(&set,&rcvdsig);
+            if (rcvdsig == SIGCHLD) {
+                if (wait(&err) < 0) {
+                    perror("Wait failed");
+                    exit (1);
+                }
+                if (WIFEXITED(err))
+                    exit(WEXITSTATUS(err));
 
-        /* Continue here as child */
+                exit(1);
+            }
+            exit(0);
+        }
+    }
+
+    /* Continue as child */
+
+    sigprocmask(SIG_SETMASK,&oset,NULL);
+
+    if (pidfile) {
+        /* Check for pidfile and lock it if not locked already */
+        if ((pfd=open(pidfile,O_RDWR|O_CREAT,0644)) < 0) {
+            fprintf(stderr,"Could not create pid file: %s",strerror(errno));
+            exit(1);
+        }
+
+        if ((fl=(struct flock *) malloc(sizeof(struct flock))) == NULL) {
+            perror("Could not allocate memory for pid lock");
+            exit(1);
+        }
+
+        fl->l_type=F_WRLCK;
+        fl->l_whence=SEEK_SET;
+        fl->l_start=0;
+        fl->l_len=0;
+
+        if ((err=fcntl(pfd,F_SETLK,fl)) < 0) {
+            if (errno == EACCES || err == EAGAIN ) {
+                fprintf(stderr,"pid file %s currently lock by pid %d\n",pidfile,
+                        (int) fl->l_pid); 
+            } else {
+                fprintf(stderr,"Could not lock pid file %s: %s\n",pidfile,
+                        strerror(err));
+            }
+            exit(1);
+        }
+        if (truncate(pidfile,0) < 0) {
+            fprintf(stderr,"Could not truncate pid file %s: %s\n",pidfile,
+                    strerror(errno));
+            exit(1);
+        }
+
+        dprintf(pfd,"%d",getpid());
+    }
+
+    if (ifg->flags & K_BACKGROUND) {
 
         /* Really should close all file descriptors. Harder to do in OS
          * independent way.  Just close the ones we know about for this cut
@@ -1436,6 +1560,10 @@ int main(int argc, char ** argv)
             fclose(stderr);
             ifg->flags |= K_NOSTDERR;
         }
+
+        /* Tell Parent it's OK to exit */
+        kill(getppid(),SIGUSR1);
+
         setsid();
         (void) chdir("/");
         umask(0);
@@ -1452,7 +1580,7 @@ int main(int argc, char ** argv)
     if (getrlimit(RLIMIT_NOFILE,&lim) < 0)
             logterm(errno,"Couldn't get resource limits");
     if (lim.rlim_cur > 1<<IDMINORBITS) {
-        logwarn("Lowering NOFILE from %u to %u",lim.rlim_cur,1<<IDMINORBITS);
+        DEBUG(3,"Lowering NOFILE from %u to %u",lim.rlim_cur,1<<IDMINORBITS);
         lim.rlim_cur=1<<IDMINORBITS;
         if(setrlimit(RLIMIT_NOFILE,&lim) < 0)
             logterm(errno,"Could not set file descriptor limit");
@@ -1509,8 +1637,13 @@ int main(int argc, char ** argv)
 
             if (ifptr->checksum <0)
                 ifptr->checksum = engine->checksum;
-            if (ifptr->strict <0)
-                ifptr->strict = engine->strict;
+            if (ifptr->strict <0) {
+                if (engine->strict >= 0) {
+                    ifptr->strict = engine->strict;
+                } else {
+                    ifptr->strict = (ifptr->type == FILEIO)?0:1;
+                }
+            }
             (*tiptr)=ifptr;
             tiptr=&ifptr->next;
             if (ifptr->next==ifptr2)
@@ -1522,7 +1655,7 @@ int main(int argc, char ** argv)
      * mapping so we can update references to "name" with an id
      */
     for (ifptr=lists.initialized;ifptr;ifptr=ifptr->next) {
-        if (ifptr->ofilter)
+        if (ifptr->direction != IN && ifptr->ofilter)
             if (name2id(ifptr->ofilter))
                 logterm(errno,"Name to interface translation failed");
     }
@@ -1607,6 +1740,8 @@ int main(int argc, char ** argv)
              */
             (void) sigwait(&set,&rcvdsig);
             pthread_mutex_lock(&lists.io_mutex);
+        } else {
+            rcvdsig = 0;
         }
 
         if ((timetodie > 0) || ( lists.outputs == NULL && (timetodie == 0)) ||
@@ -1650,6 +1785,10 @@ int main(int argc, char ** argv)
     /* For neatness... */
     pthread_mutex_unlock(&lists.io_mutex);
 
+    DEBUG(3,"Removing pid file");
+
+    unlink(pidfile);
+
     DEBUG(1,"Kplex exiting");
 
     exit(0);
