@@ -165,7 +165,7 @@ int reconnect(iface_t *ifa, int err)
 
     DEBUG(3,"%s: Reconnecting (write) interface",ifa->name);
 
-    /* ift->shared_t_mutex shoudl be locked by the calling routine */
+    /* ift->shared_t_mutex should be locked by the calling routine */
 
     /* If the write timed out, we don't need to sleep before retrying */
     switch (err) {
@@ -224,7 +224,6 @@ int reconnect(iface_t *ifa, int err)
     DEBUG(7,"Flushing queue interface %s",ifa->name);
     flush_queue(ifa->q);
 
-    pthread_mutex_unlock(&ift->shared->t_mutex);
     return(retval);
 }
 
@@ -577,14 +576,17 @@ iface_t *new_tcp_conn(int fd, iface_t *ifa)
     int on=1;
     sigset_t set,saved;
 
-    if ((newifa = malloc(sizeof(iface_t))) == NULL)
+    if ((newifa = malloc(sizeof(iface_t))) == NULL) {
+        logerr(errno,"malloc failed for %s",ifa->name);
         return(NULL);
+    }
 
     memset(newifa,0,sizeof(iface_t));
 
     if (((newift = (struct if_tcp *) malloc(sizeof(struct if_tcp))) == NULL) ||
             ((ifa->direction != IN) &&
             (init_q(newifa, oldift->qsize) < 0))) {
+        logerr(errno,"Failed to set up new connection");
         if (newifa && newifa->q)
             free(newifa->q);
         if (newift)
@@ -659,8 +661,11 @@ void tcp_server(iface_t *ifa)
     if (listen(ift->fd,5) == 0) {
         while(ifa->direction != NONE) {
             slen = sizeof(struct sockaddr_storage);
-            if ((afd = accept(ift->fd,(struct sockaddr *) &sad,&slen)) < 0)
-                break;
+            if ((afd = accept(ift->fd,(struct sockaddr *) &sad,&slen)) < 0) {
+                afd=errno;
+                logerr(errno,"accept failed for connection to %s",ifa->name);
+                continue;
+            }
     
             if ((newifa = new_tcp_conn(afd,ifa)) == NULL) {
                 close(afd);
@@ -724,7 +729,7 @@ struct tcp_preamble *parse_preamble(const char * val)
                         return(NULL);
                     tval<<=4;
                     if (*optr >= '0' && *optr <= '9')
-                        tval += *ptr - '0';
+                        tval += *optr - '0';
                     else if (*optr >= 'a' && *optr <= 'f')
                         tval += (*optr - 'a' + 10);
                     else if (*optr >= 'A' && *optr <= 'F')
@@ -738,8 +743,10 @@ struct tcp_preamble *parse_preamble(const char * val)
                 return(NULL);
             default:
                 for (i=0,tval=0;i<3;i++) {
-                    tval <<=3;
-                    ++optr;
+                    if (i) {
+                        ++optr;
+                        tval <<=3;
+                    }
                     if (*optr >= '0' && *optr <= '7')
                         tval += (*optr - '0');
                     else if (i == 0) {
@@ -749,12 +756,11 @@ struct tcp_preamble *parse_preamble(const char * val)
                         return(NULL);
                 }
                 if (i == 3) {
-                    if (tval < 256)
+                    if (tval < 512)
                         *ptr++ = (unsigned char) tval;
                     else
                         return(NULL);
-                } else
-                    *ptr++ = *optr++;
+                }
                 break;
             }
         } else
@@ -813,7 +819,7 @@ iface_t *init_tcp(iface_t *ifa)
         return(NULL);
     }
 
-    ift->qsize=DEFTCPQSIZE;
+    ift->qsize=DEFQSIZE;
     ift->shared=NULL;
     preamble=NULL;
 
